/* Einfaches Beispiel fuer die Anwendung von Swing und AWT.
Demonstration der Model-View-Controller-Architektur.
Vorlesung "Softwaretechnologie".
(c) Heinrich Hussmann, TU Dresden 1997-2000 */

import java.util.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class WindowCloser extends WindowAdapter implements WindowListener {

	 public void windowClosing(WindowEvent event) {
	 	System.exit(0);
	 }
}

// The View

class CounterFrame extends JFrame implements Observer {

	JPanel valuePanel = new JPanel();
	JTextField valueDisplay = new JTextField(10);

	JButton countButton = new JButton("Count");
	JButton resetButton = new JButton("Reset");
	JButton exitButton = new JButton("Exit");

	JPanel buttonPanel = new JPanel();

	public CounterFrame (Counter c) {
	 	setTitle("Counter");

	 	valuePanel.add(new Label("Counter value"));
	 	valuePanel.add(valueDisplay);
	 	valueDisplay.setEditable(false);
	 	valueDisplay.setText(String.valueOf(c.getValue()));
	 	getContentPane().add(valuePanel,BorderLayout.NORTH);

	 	ButtonController buttonContr = new ButtonController(c);
	 	countButton.addActionListener(buttonContr);
	 	buttonPanel.add(countButton);
	 	resetButton.addActionListener(buttonContr);
	 	buttonPanel.add(resetButton);
	 	exitButton.addActionListener(buttonContr);
	 	buttonPanel.add(exitButton);
	 	getContentPane().add(buttonPanel,BorderLayout.SOUTH);

	 	addWindowListener(new WindowCloser());
	 	c.addObserver(this);

	 	pack();
	 	setVisible(true);
	 }

	 public void update (Observable o, Object arg) {
	 	Counter c = (Counter) o;
	 	valueDisplay.setText(String.valueOf(c.getValue()));
	 }
}

// The Controller

class ButtonController implements ActionListener {

	Counter myCounter;

	public void actionPerformed (ActionEvent event) {
		String cmd = event.getActionCommand();
		if (cmd.equals("Count"))
			myCounter.count();
		if (cmd.equals("Reset"))
			myCounter.reset();
		if (cmd.equals("Exit"))
			System.exit(0);
	}

	public ButtonController (Counter c) {
		myCounter = c;
	}
}

// The Model

class Counter extends Observable {

	private int k = 0;

	public void count () {
		k++;
		setChanged();
		notifyObservers();
	}

	public void reset () {
		k = 0;
		setChanged();
		notifyObservers();
	}

	public int getValue () {
		return k;
	}
}

class MVCx {

public static void main (String[] argv) {
  // Look and Feel kann durch Angabe eines (beliebigen) Arguments auf der Kommandozeile umgeschaltet werden
  // Also z.B. "java MVCx test liefert anderes Look And Feel"
    try {
      if (argv.length == 0)
   	UIManager.setLookAndFeel(
    	  UIManager.getCrossPlatformLookAndFeelClassName());
      else
        UIManager.setLookAndFeel(
          UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {}
	CounterFrame cf = new CounterFrame(new Counter());
	}
}
